//	This file is part of City Growth Limiter Minimal by gnrd
//	Forked from City Growth Limiter v6 by Sylf
//
//	Copyright (C) 2014 Sylf
//	Copyright (C) 2015 gnrd
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License along
//	with this program; if not, write to the Free Software Foundation, Inc.,
//	51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

require("town.nut");

class CiGrLiMin extends GSController 
{
	constructor() 
	{
	}
}

function CiGrLiMin::Start() 
{	
	// Town growth rate must be non zero
	if (GSGameSettings.IsValid("town_growth_rate")) {
		if (!GSGameSettings.GetValue("town_growth_rate")) {
			GSLog.Error("Town growth is deactivated. Script terminating...");
			return;
		}
	}
	
	// Ckeck PAX cargo ID
	if (!GSCargo.IsValidCargo(0)) {
		GSLog.Error("Default PAX cargo ID is invalid. Script terminating...");
		return;
	}
	if (!GSCargo.HasCargoClass(0,  GSCargo.CC_PASSENGERS)) {
		GSLog.Error("PAX cargo ID is different from default. Script terminating...");
		return;
	}
	
	// Disallow founding towns
	if (GSGameSettings.IsValid("economy.found_town")) GSGameSettings.SetValue("economy.found_town", 0);
	
	// Create the towns list
	local towns = this.CreateTownList();
	
	// Check town number - on a newly generated map, with this implementation, each
	// city in the foreach cycle of the main loop takes 2 ticks (Sleep command included)
	local i = 0;
	foreach (town in towns) i++;
	GSLog.Info("Number of towns: " + i);
	if (i < 2) {
		GSLog.Error("Not enough towns for PAX game! Script terminating");
		return;
	}
	if ((i > 1036) && (i <= 2072)) GSLog.Warning("Town number very high. Growth management will have a longer cycle.");
	if (i > 2072) {
		GSLog.Error("Town number too high for the script to work as intended.");
		if (!GSController.GetSetting("allow_crowded_map")) return;
	}
	
	// OK here we go
	i = null;
	local j = null;
	GSLog.Info("Cycle starting...");
	this.Sleep(1); // Wait for World Gen end?
	
	while(true) {
		i = this.GetTick();
		foreach (town in towns) {
			town.ManageTown();
			this.Sleep(1);
		}
		j = this.GetTick() - i;
		this.Sleep(max(1, 2072 - j));  
	}
}

// This "growth management cycle" should be around 28 days
// long (28*74 ticks) for number of towns == 1036 but
// it may later end up taking more time. That's why
// we check how many ticks have been used every time -
// so it can be handled for towns < 1036. If
// towns > 1036 the cycle just gets longer...

function CiGrLiMin::CreateTownList()
{
	local result = [];
	// Make a squirrel array of all towns.
	// For each town, an instance of Town is created
	// to store the data related to that town
	local all_towns = GSTownList();
	foreach(t, _ in all_towns)
	{
		result.append(Town(t));
	}
	return result;
}

//function CiGrLiMin::Save()
//{
//	
//}

//function CiGrLiMin::Load()
//{
//	
//}
